/*********************************************************************************************
	MACRO NAME: read
	PURPOSE: Read in IPUMS data
*********************************************************************************************/

* BEGIN MACRO CODE;
%macro macro_read_ipum;

* pull needed variables (CUSTOM);
data ipums;
	length cpsidp cpsid $14 /*marbasecidp*/ marbasecidh $10 hrhhid $15 hrhhid2 $5;
	set &ipumlib..&ipums. (keep=year month mish hrhhid hrhhid2 cpsidp cpsid pernum lineno uh_relhd_b4 relate wtfinl hwtfinl popstat 
		&basicwide. &basicfirstp. &basicfirsth. &asecfirst.
		asecflag /*marbasecidp*/ marbasecidh 
		where=(popstat=1 /*and pernum=1*/) 
		rename=(cpsidp=cpsidp_num cpsid=cpsid_num 
				/*marbasecidp=marbasecidp_num*/ marbasecidh=marbasecidh_num 
				hrhhid=hrhhid_num hrhhid2=hrhhid2_num));
	* convert cpsid to string;
	format cpsidp_num cpsid_num 14. /*marbasecidp_num*/ marbasecidh_num 10. hrhhid_num 15. hrhhid2_num 5.;
	cpsidp=trim(put(cpsidp_num,$14.));
	cpsid=trim(put(cpsid_num,$14.));
	/*marbasecidp=trim(put(marbasecidp_num,$10.));*/
	marbasecidh=trim(put(marbasecidh_num,$10.));
	hrhhid=trim(put(hrhhid_num,$15.));
	hrhhid2=trim(put(hrhhid2_num,$5.));
	* months since 2000 (2008 Jan=8*12+1=97. 2009 Jan=9*12+1=109. 2010 Jan=10*12+1=121);
	mtime=(year-2000)*12+month;
run;
proc freq data=ipums;
	title "Years and Months in sample";
	table year*month mtime /missing;
run;

* remove duplicates and separate by survey, need to investigate duplicates;
proc sort data=ipums (where=(asecflag ne 1)) out=ipumsbasic nodupkey;
	by cpsidp year month;
run;
* remove those in asec who do not link to cps, could also drop those with substr(marbasecidh,1,2)=11;
proc sort data=ipums (where=(asecflag=1 and cpsidp_num ne 0)) out=ipumsasec nodupkey;
	by cpsidp year month;
run;

* create variable for when first observed, use for defining samples;
data ipumsfirst (keep=start_year start_month start_mish start_mtime start_mtime_adj start_hrhhid start_hrhhid2 cpsidp cpsid start_pernum start_lineno start_uh_relhd_b4 start_relate start_wtfinl start_hwtfinl rand);
	set ipumsbasic (keep=year month mish hrhhid hrhhid2 cpsidp cpsid pernum lineno uh_relhd_b4 relate wtfinl hwtfinl);
	by cpsidp year month;
	if (first.cpsidp) then do;
		start_year=year;
		start_month=month;
		start_mish=mish;
		start_mtime=(start_year-2000)*12+start_month;
		if (start_mish<=4) then start_mtime_adj=start_mtime-(start_mish-1);
		else start_mtime_adj=start_mtime-(start_mish-5+12);
		start_hrhhid=hrhhid;
		start_hrhhid2=hrhhid2;
		start_pernum=pernum;
		start_lineno=lineno;
		start_uh_relhd_b4=uh_relhd_b4;
		start_relate=relate;
		* 14-digit numeric variable with four implied decimals. That is, 12345678901234 should be interpreted as 1234567890.1234;
		start_wtfinl=wtfinl/10000;
		* household-level weight that should be used to generate statistics about households;
		start_hwtfinl=hwtfinl/10000;
		* random draw in case need to subset;
		rand=ranuni(10);
		output;
	end;
run;
proc freq data=ipumsfirst;
	title "First observed";
	table start_mtime start_mtime_adj start_mish /missing;
run;
proc means data=ipumsfirst mean min max sum;
	title "Weights, all obs";
    var start_wtfinl start_hwtfinl;
run;
proc means data=ipumsfirst (where=(start_mish=1)) mean min max sum;
	title "Weights, start in first MIS";
	var start_wtfinl start_hwtfinl;
run;

*END MACRO CODE;
%mend macro_read_ipum;

* BEGIN MACRO CODE;
%macro macro_read_first;

* limit to in-scope persons, starting sample within extract (CUSTOM);
data ipumsfirst;
	set ipumsfirst (where=(start_mtime_adj in (&validmtimelist.)));
run;
proc freq data=ipumsfirst;
	title "First observed, starting in extract range";
	table start_mtime start_mtime_adj start_mish /missing;
run;

* merge in-scope persons with months frame - full interaction (CUSTOM);
proc sql nowarn;
	create table ipumsmonth as
	select *
	from ipumsfirst, months_&samp.
	;
quit;
proc freq data=ipumsmonth;
	title "Check series, before retaining valid";
	table start_mtime_adj*series /missing;
run;
* retain only the months in the appropriate series;
data ipumsmonth;
	set ipumsmonth;
	* loop through valid start mtime and series combinations, only output valid;
	* need quote because of commas in list;
	%let startord=1;
	%do %until("%scan(%quote(&validmtimelist.),&startord.)"="");
		%let validmtime=%scan(%quote(&validmtimelist.),&startord.);
		%let validseries=%scan(%quote(&validserieslist.),&startord.);
		if (start_mtime_adj=&validmtime. and series=&validseries.) then output ipumsmonth;
		%let startord=%eval(&startord.+1);
	%end;
run;
proc freq data=ipumsmonth;
	title "Check series, after retaining valid";
	table start_mtime_adj*series /missing;
run;
proc sort data=ipumsmonth;
	by cpsidp year month;
run;
proc print data=ipumsmonth (obs=20);
	title "Check merge to months";
run;

*END MACRO CODE;
%mend macro_read_first;

* BEGIN MACRO CODE;
%macro macro_read_person;

* add person data by month, where available (do not retain out-of-scope persons);
data ipumsmonth;
	merge ipumsmonth (in=inmonth) 
		ipumsbasic (in=inbasic keep=cpsidp year month mish &basicwide. &basicfirstp. &basicfirsth.)
		ipumsasec (in=inasec keep=cpsidp year month &asecfirst.)
		;
	by cpsidp year month;
	if (inmonth);
	if (inbasic) then hasbasic=1;
	else hasbasic=0;
	if (inasec) then hasasec=1;
	else hasasec=0;
	mtime=(year-2000)*12+month;
run;
proc sql;
   create table distinct as 
     select count(distinct(cpsidp)) as cpsidpcount,
            count(distinct(mtime)) as mtimecount
            from ipumsmonth;
quit;
proc print data=distinct;
   title 'ipumsmonth Number of distinct values for each variable'; 
run;
proc contents data=ipumsmonth;
	title "Monthly data";
run;
proc freq data=ipumsmonth;
	table start_mtime_adj*mish start_mtime_adj*seq start_mtime_adj*hasasec /missing;
run;

* output monthly file (CUSTOM);
data DATAWORK.monthly_&samp.;
	set ipumsmonth;
run;

*END MACRO CODE;
%mend macro_read_person;
